package controller;

import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Alert;
import javafx.scene.control.ButtonType;
import javafx.scene.control.ChoiceDialog;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Polygon;
import model.GameState;
import model.Player;
import model.Tile;
import view.TileView;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

public class Controller implements Initializable
{

	private GameState gameState;
	private Process gameProcess;
	private PrintWriter gameInput;
	private Tile[] board;
	private TileView[] boardView;
	private Player humanPlayer;
	private int selectedTile;

	@FXML
	private Polygon tile0, tile1, tile2, tile3, tile4, tile5, tile6, tile7, tile8, tile9, tile10, tile11, tile12, tile13, tile14, tile15, tile16, tile17, tile18, tile19, tile20, tile21, tile22, tile23, tile24, tile25, tile26, tile27, tile28, tile29, tile30, tile31, tile32, tile33, tile34, tile35, tile36, tile37, tile38, tile39, tile40, tile41, tile42, tile43, tile44, tile45, tile46, tile47, tile48, tile49, tile50, tile51, tile52, tile53, tile54, tile55, tile56, tile57, tile58, tile59;

	@FXML
	private BorderPane mainPane;

	@Override
	public void initialize(URL location, ResourceBundle resources)
	{
		Polygon[] fxTiles = {tile0, tile1, tile2, tile3, tile4, tile5, tile6, tile7, tile8, tile9, tile10, tile11, tile12, tile13, tile14, tile15, tile16, tile17, tile18, tile19, tile20, tile21, tile22, tile23, tile24, tile25, tile26, tile27, tile28, tile29, tile30, tile31, tile32, tile33, tile34, tile35, tile36, tile37, tile38, tile39, tile40, tile41, tile42, tile43, tile44, tile45, tile46, tile47, tile48, tile49, tile50, tile51, tile52, tile53, tile54, tile55, tile56, tile57, tile58, tile59};
		this.board = new Tile[fxTiles.length];
		this.boardView = new TileView[fxTiles.length];
		for (int i = 0; i < fxTiles.length; i++)
		{
			board[i] = new Tile(i, Tile.PenguinPresence.NO_PENGUIN);
			fxTiles[i].setOnMouseClicked(new TileClickHandler(i));
			boardView[i] = new TileView(fxTiles[i], board[i]);
		}
		this.selectedTile = -1;
		this.gameState = new GameState();
		try
		{
			this.gameProcess = new ProcessBuilder("../bin/theturk").start();
			gameInput = new PrintWriter(new OutputStreamWriter(gameProcess.getOutputStream()), true);
			Runtime.getRuntime().addShutdownHook(new Thread()
			{
				public void run()
				{
					gameProcess.destroy();
				}
			});
		}
		catch (IOException e)
		{
			Alert alert = new Alert(Alert.AlertType.ERROR, "Can't run penguin program", ButtonType.OK);
			alert.showAndWait();
			e.printStackTrace();
			System.exit(1);
		}
		UpdateThread upT = new UpdateThread(gameProcess, this.gameState, this.board, this.boardView);
		upT.setDaemon(true);
		upT.start();

		//Choice: who will start?
		ChoiceDialog<Player> playerChoice = new ChoiceDialog<>(Player.Red, Player.Red, Player.Blue);
		playerChoice.setTitle("Penguin game!");
		playerChoice.setHeaderText("Choose your color (red starts)");
		playerChoice.showAndWait();
		humanPlayer = playerChoice.getResult();
		if (humanPlayer == null)
		{
			System.exit(0);
		}
		else
		{
			if (humanPlayer.equals(Player.Red))
				gameInput.println("h");
			else
				gameInput.println("c");
		}
	}

	/**
	 * Event handler that will listen for a click on a tile.
	 * It is associated with a TileView
	 */
	private class TileClickHandler implements EventHandler<MouseEvent>
	{
		private int tileNumber;

		public TileClickHandler(int tileNumber)
		{
			this.tileNumber = tileNumber;
		}

		/**
		 * Change the tiles' highlighted values to the given value
		 */
		private void changeTilesHighlightState(List<Integer> tiles, boolean value)
		{
			for (int tile : tiles)
			{
				boardView[tile].setHighlighted(value);
				boardView[tile].update();
			}
		}

		@Override
		public void handle(MouseEvent event)
		{
			if (!gameState.getCurrent_player().equals(humanPlayer))
				return;

			if (selectedTile == tileNumber) //If we clicked again on the selected tile
			{
				//UnSelect and un-highlight previously selected and highlighted tiles
				boardView[selectedTile].setSelected(true);
				boardView[selectedTile].update();
				changeTilesHighlightState(gameState.getReachableTiles(humanPlayer, gameState.getPenguinOnTile(humanPlayer, selectedTile)), false);
				selectedTile = -1;
				return;
			}

			//We clicked on a tile that wasn't previously selected
			int penguinNb = gameState.getPenguinOnTile(humanPlayer, tileNumber);
			if (penguinNb == -1) //There is no penguin on this tile
				return;

			//There is a penguin on the tile: we want to select this tile
			if (selectedTile != -1) //If there was a tile previously selected
			{
				boardView[selectedTile].setSelected(false);
				boardView[selectedTile].update();
				changeTilesHighlightState(gameState.getReachableTiles(humanPlayer, gameState.getPenguinOnTile(humanPlayer, selectedTile)), false);
			}
			boardView[tileNumber].setSelected(true);
			boardView[tileNumber].update();
			changeTilesHighlightState(gameState.getReachableTiles(humanPlayer, penguinNb), true);
			selectedTile = tileNumber;
			return;
		}
	}

	private class MyHexEnteredHandler implements EventHandler<MouseEvent>
	{
		@Override
		public void handle(MouseEvent event)
		{
			Polygon poly = (Polygon) event.getSource();
			poly.setFill(Color.GOLD);
		}
	}


}
